/* $Id: naco_img_lingain.c,v 1.9 2008/08/22 11:46:46 llundin Exp $
 *
 * This file is part of the DETMON Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2008/08/22 11:46:46 $
 * $Revision: 1.9 $
 * $Name: HEAD $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*----------------------------------------------------------------------------
                             Includes and Defines
 ----------------------------------------------------------------------------*/

#include <cpl.h>

#include "irplib_detmon_lg.h"
#include "irplib_plugin.h"

/* Define here the DO.CATG keywords */
#define NACO_IMG_LINGAIN_ON_RAW             "CAL_LINGAIN_LAMP"
#define NACO_IMG_LINGAIN_OFF_RAW            "CAL_LINGAIN_DARK"

#define RECIPE_NAME "naco_img_lingain"

/* Copy here instrument specific keywords which need to be in the PAF file */
#define INSTREGEXP   "ESO DET MODE NAME|ESO DET NCORRS NAME|" \
                     "ESO INS OPTI7 NAME|ESO INS OPTI6 NAME|" \
                     "ESO INS OPTI5 NAME|ESO INS OPTI4 NAME"

#define PAFREGEXP    "^(" REGEXP "|" INSTREGEXP ")$"

#define NIR TRUE

#ifndef NACO_REF_LEVEL
/* DFO states that the NACO reference level is in the range 1600 - 4000
   with 3500 being the typical level. For one dataset 3500 is too high,
   while values below 2300 work. */
#define NACO_REF_LEVEL 2000
#endif

/*----------------------------------------------------------------------------
                 Functions prototypes
 ----------------------------------------------------------------------------*/
static cpl_error_code naco_img_lingain_fill_parameterlist(cpl_parameterlist *);

CPL_RECIPE_DEFINE(naco_img_lingain, NACO_BINARY_VERSION,
                  naco_img_lingain_fill_parameterlist(recipe->parameters),
                  "Lars Lundin", PACKAGE_BUGREPORT, "2007", 
                  "Linearity/Gain recipe for the IR domain",
                  irplib_detmon_lg_get_description(RECIPE_NAME, "NACO",
                                                   NACO_IMG_LINGAIN_ON_RAW,
                                                   NACO_IMG_LINGAIN_OFF_RAW));

/*----------------------------------------------------------------------------*/
/**
 * @defgroup naco_img_lingain   Linearity/Gain
 */
/*----------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
                                Functions code
 -----------------------------------------------------------------------------*/



/*---------------------------------------------------------------------------*/
/*
  @brief    Interpret the command line options and execute the data processing
  @param    frameset    the frames list
  @param    parlist     the parameters list
  @return   0 iff everything is ok
 */
/*---------------------------------------------------------------------------*/
static int naco_img_lingain(cpl_frameset            * frameset,
                            const cpl_parameterlist * parlist)
{

    cpl_propertylist * pro_lintbl     = cpl_propertylist_new();
    cpl_propertylist * pro_gaintbl    = cpl_propertylist_new();
    cpl_propertylist * pro_coeffscube = cpl_propertylist_new();
    cpl_propertylist * pro_bpm        = cpl_propertylist_new();
    cpl_propertylist * pro_corr       = cpl_propertylist_new();
    cpl_propertylist * pro_diff       = cpl_propertylist_new();
    cpl_error_code     error = CPL_ERROR_NONE;

    error |= cpl_propertylist_append_string(pro_lintbl, "ESO PRO CATG",
                                            "DET_LIN_INFO");
    error |= cpl_propertylist_append_string(pro_gaintbl, "ESO PRO CATG",
                                            "GAIN_INFO");
    error |= cpl_propertylist_append_string(pro_coeffscube, "ESO PRO CATG",
                                            "COEFFS_CUBE");
    error |= cpl_propertylist_append_string(pro_bpm, "ESO PRO CATG",
                                            "BP_MAP_NL");
    error |= cpl_propertylist_append_string(pro_corr, "ESO PRO CATG",
                                            "AUTOCORR");
    error |= cpl_propertylist_append_string(pro_diff, "ESO PRO CATG",
                                            "DIFF_FLAT");

    error |= irplib_detmon_lg(frameset,
                              parlist,
                              NACO_IMG_LINGAIN_ON_RAW,
                              NACO_IMG_LINGAIN_OFF_RAW,
                              RECIPE_NAME,
                              PACKAGE,
                              PAFREGEXP,
                              pro_lintbl,
                              pro_gaintbl,
                              pro_coeffscube,
                              pro_bpm,
                              pro_corr,
                              pro_diff,
                              PACKAGE "/" PACKAGE_VERSION,
                              NULL, NULL, NIR);

    cpl_propertylist_delete(pro_lintbl);
    cpl_propertylist_delete(pro_gaintbl);
    cpl_propertylist_delete(pro_coeffscube);
    cpl_propertylist_delete(pro_bpm);
    cpl_propertylist_delete(pro_corr);
    cpl_propertylist_delete(pro_diff);

    /* Propagate the error, if any */
    cpl_ensure_code(!error, cpl_error_get_code());

    return CPL_ERROR_NONE;
}



/*---------------------------------------------------------------------------*/
/*
  @internal
  @brief    Fill the parameterlist including NACO specific defaults
  @param    self     The parameter list to fill
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/
static
cpl_error_code naco_img_lingain_fill_parameterlist(cpl_parameterlist * self)
{

    cpl_error_code error =
        irplib_detmon_lg_fill_parlist_nir_default(self, RECIPE_NAME, PACKAGE);
    cpl_parameter * p;

    cpl_ensure_code(!error, error);

    /* FIXME: Put the parameter names and their new default values in two arrays
       and iterate through them */
    p = cpl_parameterlist_find(self, PACKAGE "." RECIPE_NAME ".ref_level");

    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);

    error = cpl_parameter_set_default_int(p, NACO_REF_LEVEL);

    cpl_ensure_code(!error, error);

    return CPL_ERROR_NONE;
}
